package tapl3

trait TestSuite {
  type Output
  def parse(in: String): Option[Output]

  def reduce(t: Output): Output
  def reduceWithCallByValue(t: Output): Output
  def path(t: Output, reduce: Output => Output): Stream[Output]

  type Context = List[Pair[String, Type]]
  private val ctx: Context = Nil

  def typeof(ctx: Context, t: Output): Type
  def typeof(t: Output): Type = typeof(ctx, t)

  case class TypeException(msg: String) extends Exception(msg)

  def checkString(line: String) = {
    Console.println("Reduce path:\n" + line)
    val Some(t) = parse(line)
    try {
      Console.println("Type is: " + typeof(t))
      run("Full beta reduction", path(t, reduce))
      run("Call by value", path(t, reduceWithCallByValue))
    } catch {
      case TypeException(msg) =>
        Console.println("Type error: " + msg)
    }
    Console.println("-------------------------------\n")
  }

  def check(filename: String) = {
    val buf = new compat.StringBuilder
    io.Source.fromFile(filename) foreach (line => buf.append(line))
    checkString(buf.toString)
  }

  private def run(msg: String, s: Stream[Output]) {
    Console.println("\n" + msg + ":")
    for (val t <- s) Console.println(t)
  }

}
